#ifndef __LINUX_SPINLOCK_TYPES_H
#define __LINUX_SPINLOCK_TYPES_H

/*
 * include/linux/spinlock_types.h - generic spinlock type definitions
 *                                  and initializers
 *
 * portions Copyright 2005, Red Hat, Inc., Ingo Molnar
 * Released under the General Public License (GPL).
 */

#if defined(CONFIG_SMP)
# include <asm/spinlock_types.h>
#else
# include <linux/spinlock_types_up.h>
#endif

typedef struct {
	raw_spinlock_t raw_lock;
#if defined(CONFIG_PREEMPT) && defined(CONFIG_SMP)
	unsigned int break_lock;
#endif
#ifdef CONFIG_DEBUG_SPINLOCK
	unsigned int magic, owner_cpu;
	void *owner;
#endif
} spinlock_t;

#define SPINLOCK_MAGIC		0xdead4ead

typedef struct {
	raw_rwlock_t raw_lock;
#if defined(CONFIG_PREEMPT) && defined(CONFIG_SMP)
	unsigned int break_lock;
#endif
#ifdef CONFIG_DEBUG_SPINLOCK
	unsigned int magic, owner_cpu;
	void *owner;
#endif
} rwlock_t;

#define RWLOCK_MAGIC		0xdeaf1eed

#define SPINLOCK_OWNER_INIT	((void *)-1L)

#if defined(CONFIG_PREEMPT) && defined(CONFIG_SMP)
# define SPINLOCK_BREAK_LOCK_INIT 0,
#else
# define SPINLOCK_BREAK_LOCK_INIT
#endif

#ifdef CONFIG_DEBUG_SPINLOCK
# define SPIN_LOCK_UNLOCKED						\
	(spinlock_t)	{ /*raw_lock*/	__RAW_SPIN_LOCK_UNLOCKED,	\
			  /*break_lock*/ SPINLOCK_BREAK_LOCK_INIT	\
			  /*magic*/	SPINLOCK_MAGIC,			\
			  /*owner_cpu*/	-1,				\
			  /*owner*/	SPINLOCK_OWNER_INIT }
#define RW_LOCK_UNLOCKED						\
	(rwlock_t)	{ /*raw_lock*/	__RAW_RW_LOCK_UNLOCKED,		\
			  /*break_lock*/ SPINLOCK_BREAK_LOCK_INIT	\
			  /*magic*/	RWLOCK_MAGIC,			\
			  /*owner_cpu*/	-1,				\
			  /*owner*/	SPINLOCK_OWNER_INIT }
#else
# define SPIN_LOCK_UNLOCKED \
	(spinlock_t)	{ /*raw_lock*/	__RAW_SPIN_LOCK_UNLOCKED }
# define RW_LOCK_UNLOCKED \
	(rwlock_t)	{ /*raw_lock*/	__RAW_RW_LOCK_UNLOCKED }
#endif

#define DEFINE_SPINLOCK(x)	spinlock_t x = SPIN_LOCK_UNLOCKED
#define DEFINE_RWLOCK(x)	rwlock_t x = RW_LOCK_UNLOCKED

#endif /* __LINUX_SPINLOCK_TYPES_H */
